import 'package:osam/osam.dart';
import 'package:osam_flutter/src/navigation/osam_route.dart';

import '../../osam_flutter.dart';

part 'navigation.g.dart';

class NoRouteException implements Exception {}

class _NavigationUseCase extends UseCase {}

@HiveType(typeId: 222)
class Navigation {
  @HiveField(0)
  final List<OsamRoute<Object, Object>> _routes;
  final IProperty<List<OsamRoute<Object, Object>>> routes;
  final _navigationUseCase = _NavigationUseCase();

  Navigation(this._routes) : routes = IProperty(_routes);

  Future<R> push<A, R>(OsamRoute<A, R> route) {
    _navigationUseCase.updateValue(routes, (value) => value.add(route));
    return route.whenComplete;
  }

  void pop() => _navigationUseCase.updateValue(routes, (value) => value.removeLast());

  T findRoute<T extends OsamRoute>() => _routes.firstWhere((element) => element is T);

  Future<R> pushReplacement<A, R>(OsamRoute<A, R> route) {
    _navigationUseCase.updateValue(routes, (value) {
      value.removeLast();
      value.add(route);
    });
    return route.whenComplete;
  }

  Future<R> pushAndRemoveUntil<A, R>(OsamRoute<A, R> route) {
    _navigationUseCase.updateValue(routes, (value) {
      value.clear();
      value.add(route);
    });
    return route.whenComplete;
  }

  void popUntil<T extends OsamRoute>() => _navigationUseCase.updateValue(routes, (value) {
        if (value.contains(T)) {
          while (!(value.last is T)) {
            value.removeLast();
          }
        }
      });
}

@HiveType(typeId: 221)
class TabNavigation {
  @HiveField(0)
  String _tab;
  final IProperty<String> tabNotifier;
  final _navigationUseCase = _NavigationUseCase();

  TabNavigation(this._tab) : tabNotifier = IProperty(_tab);

  set tab(String tab) {
    _tab = tab;
    _navigationUseCase.setValue(tabNotifier, _tab);
  }
}
